# use Miniconda environment
#author = "Kang Yang"
#copyright = "Copyright 2017, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "plot lake/river information and all SMB models"

import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import pysal as ps
from matplotlib import cm
from textwrap import wrap

def create_waterarea_fraction_DF_for_all_elevations(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterareaIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterarea = df.loc[waterareaIdx]

    watersum = waterarea.sum(axis = 1)


    areasum = area.sum()

    resolution = 30
    waterfraction = watersum*resolution*resolution/(1000000*areasum)


    waterfraction.index = [int(day) for day in waterareaIdx]
    waterfraction = waterfraction.sort_index(ascending=True)

    return waterfraction

def calculate_SMB_runoff_for_all_elevation(df_smb,df_area_bin):

    df_smb = df_smb * df_area_bin * 1000 / (24*3600)  #unit: dm_smb, mm/day; df_area, km2, convert the unit into m3/s

    return df_smb

def create_all_SMB_DF(input_smb_file,area):
    data = pd.read_excel(input_smb_file)
    df = pd.DataFrame(data)
    df['DOY'] = df.index
    cols = list(df)
    mar_cols = [c for c in cols if 'mar' in c]
    racmo_cols = [c for c in cols if 'racmo' in c]
    merra_cols = [c for c in cols if 'merra' in c]

    mar_cols.insert(0,'DOY')
    racmo_cols.insert(0,'DOY')
    merra_cols.insert(0,'DOY')

    df_mar = df[mar_cols]
    df_racmo = df[racmo_cols]
    df_merra = df[merra_cols]
    df_mar.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_racmo.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_merra.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_mar.set_index('DOY', inplace=True)
    df_racmo.set_index('DOY', inplace=True)
    df_merra.set_index('DOY', inplace=True)

    df_mar = df_mar /(area * 1000000)*3600*1000  #chagne m3/s/day into mm/day, note, 24 hour day has been added previously
    df_racmo = df_racmo /(area * 1000000)*3600*1000
    df_merra = df_merra /(area * 1000000)*3600*1000

    return df_mar,df_racmo,df_merra

def get_eleBin_area(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    area = df.loc[areaIdx]

    return area

def get_axis_limits(ax, scale1=0.0,scale2=0.95):
    return ax.get_xlim()[1]*scale1, ax.get_ylim()[1]*scale2

def create_waterarea_fraction_DF(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterareaIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterarea = df.loc[waterareaIdx]

    resolution = 30
    waterfraction = waterarea*resolution*resolution/(1000000*area.values)
    waterfraction.index = [int(day) for day in waterareaIdx]
    waterfraction = waterfraction.sort_index(ascending=True)
    return waterfraction

def create_watervolume_DF(input_zonalSta_watervolume_point_file):
    dbf = ps.open(input_zonalSta_watervolume_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterdepthIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterdepth = df.loc[waterdepthIdx]

    resolution = 30
    overestimation_rate = 0.8  #according to Pope, the spectral method overestimates water, use this coefficient to rectify the volume
    waterVolume = waterdepth * resolution * resolution / (1000000*area.values) * 1000 * overestimation_rate# water volume in mm
    waterVolume.index = [int(day) for day in waterdepthIdx]
    waterVolume = waterVolume.sort_index(ascending=True)
    return waterVolume

def create_SMB_DF(input_SMBExcelFile):
    filename, ext = os.path.splitext(input_SMBExcelFile)
    if ext == '.xlsx':
        data = pd.read_excel(input_SMBExcelFile)
    if ext == '.csv':
        data = pd.read_csv(input_SMBExcelFile)
    df = pd.DataFrame(data)
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    if ext == '.xlsx':
        days = df.index[:]
    if ext == '.csv':
        days = df.index[1:-1]  #In csv files, the first index is 'basin', which needs to be deleted
        df = df.iloc[1:-1]
        
    df.index = [int(day[3:]) for day in days]
    print df
    return df

def create_river_lake_DF(river_lake_stat_file):

    dbf = ps.open(river_lake_stat_file)
    d = dbf.by_col_array(dbf.header)  # my solution is better than the last sentence
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    types = df.index[:]
    areaIdx = 'area'
    lakeareaIdx = [ t for t in types if 'Larea' in t]
    lakecountIdx = [ t for t in types if 'Lcount' in t]
    riverIdx = [ t for t in types if 'Rlength' in t]

    #Use .iloc for integer based indexing and .loc for label based indexing.
    area = df.loc[areaIdx]
    lakearea = df.loc[lakeareaIdx]
    lakecount = df.loc[lakecountIdx]
    riverlength = df.loc[riverIdx]

    lakearea_dayIdx = lakearea.index[:]
    lakearea_dayIdx = [int(day[:3]) for day in lakearea_dayIdx]
    river_dayIdx = riverlength.index[:]
    river_dayIdx = [int(day[:3]) for day in river_dayIdx]

    lakefraction = lakearea/area.values
    drainage_density = riverlength/area.values

    lakecount.index = lakearea_dayIdx
    lakefraction.index = lakearea_dayIdx
    drainage_density.index = river_dayIdx

    lakecount = lakecount.sort_index(ascending=True)
    lakefraction = lakefraction.sort_index(ascending=True)
    drainage_density = drainage_density.sort_index(ascending=True)

    return lakecount,lakefraction,drainage_density

def select_doy_Runoff_from_all(df_runoff,df_all_runoff):
    doys = df_runoff.index[:]
    df_runoff = df_all_runoff.loc[doys]
    return df_runoff


#set plotting environments
matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 30})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 30
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 30
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 3
matplotlib.rcParams['lines.markersize'] = 18


#output of step 7
zonalSta_waterarea_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_waterarea.dbf'

#output of step 11
river_lake_stat_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\1_statistics\river_lake_statistics.dbf'

#output of step 13
smbPath = r'E:\1.GrIS_river_evolution\1_SMB_outputs\2015'
all_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.csv'
all_meltFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.csv'
all_nd_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.csv'


#output of step 14
runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.xlsx'
meltFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.xlsx'
nd_runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.xlsx'

#ouput of step 15
zonalSta_watervolume_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_watervolume.dbf'


lakecount,lakefraction,drainage_density = create_river_lake_DF(river_lake_stat_file)
waterfraction = create_waterarea_fraction_DF(zonalSta_waterarea_point_file)


riverfraction = waterfraction - lakefraction
riverfraction[riverfraction<0]=0
df_runoff = create_SMB_DF(runoffFile)
df_melt = create_SMB_DF(meltFile)
df_nd_runoff = create_SMB_DF(nd_runoffFile)
df_nd_runoff[df_nd_runoff<0]=0
df_all_runoff = create_SMB_DF(all_runoffFile)
df_all_melt = create_SMB_DF(all_meltFile)
df_all_nd_runoff = create_SMB_DF(all_nd_runoffFile)
df_all_nd_runoff[df_all_nd_runoff<0]=0

watervolume = create_watervolume_DF(zonalSta_watervolume_point_file)

# run E:\1.GrIS_river_evolution\00_codes\lake_filling_vs_SMB_runoff_tempuse, step 1-4 to get runoff for all SMB models
# note run SMB_Step4_calculate_smb_runoff_for_specific_days_new.py!! 2018.03.29
input_smb_file = r'E:\1.GrIS_river_evolution\00_codes\lake_filling_vs_SMB_runoff_tempuse\output_smb_runoff_output_for_DOYs.xlsx'
#output of step 13
smbPath = r'E:\1.GrIS_river_evolution\1_SMB_outputs\2015'
all_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.csv'
all_meltFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.csv'
all_nd_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.csv'
#output of step 14
runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.xlsx'
nd_runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.xlsx'
df_runoff = create_SMB_DF(runoffFile)
df_all_runoff = create_SMB_DF(all_runoffFile)
#get area for each elevation bin
areabin = get_eleBin_area(zonalSta_waterarea_point_file)
df_mar,df_racmo,df_merra = create_all_SMB_DF(input_smb_file,areabin)




#--------------------------START------------------------------#

#new contents of step 22

#calculate water fraction for all study area
zonalSta_waterarea_point_file_for_Watson = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats_for_Watson\ele_pt_waterarea.dbf'
waterfraction_allEles = create_waterarea_fraction_DF_for_all_elevations(zonalSta_waterarea_point_file_for_Watson)
waterfraction_allEles = pd.DataFrame(waterfraction_allEles, columns = ['waterfraction'])
waterfraction_allEles.index.names = ['DOY']


areabin_for_Watson = get_eleBin_area(zonalSta_waterarea_point_file_for_Watson)

df_merra=df_merra.loc[160:240]
#Note: runoff, all runoff are read from daily 7.5 km MAR models (Xavier), while df_mar is the 2015 hourly MAR data from Marco. They are very similar
df_mar = df_all_nd_runoff.loc[160:240]  
df_racmo = df_racmo.loc[160:240]

df_merra = calculate_SMB_runoff_for_all_elevation(df_merra,areabin_for_Watson)


df_mar = calculate_SMB_runoff_for_all_elevation(df_mar,areabin_for_Watson)
df_racmo = calculate_SMB_runoff_for_all_elevation(df_racmo,areabin_for_Watson)


df_merra_allEles = df_merra.sum(axis = 1)
df_mar_allEles = df_mar.sum(axis = 1)
df_racmo_allEles = df_racmo.sum(axis = 1)

print df_merra
print df_merra_allEles
print df_merra_allEles2

proglacial_dischargeFile = r'E:\1.GrIS_river_evolution\proglacial_river_discharge_from_Dirk\Watson_discharge_day_v03.csv'
proglacial_discharge_data = pd.read_csv(proglacial_dischargeFile,delim_whitespace=True)
pd_df = pd.DataFrame(proglacial_discharge_data)
pd_df = pd_df[pd_df['Year']==2015]
pd_df = pd_df[['DayOfYear','WaterFluxDivers&Temperature(m3/s)']]
pd_df.columns = ['DOY','proglacial Q']
pd_df.set_index('DOY', inplace=True)
pd_df = pd_df.loc[160:240]

f, ax1= plt.subplots()


pd_df.plot(ax = ax1,color = 'b', legend = False)
df_merra_allEles.plot(ax = ax1,color = 'g',label = 'MERRA-2')
df_mar_allEles.plot(ax = ax1,color = 'lightgreen',label = 'MAR')
df_racmo_allEles.plot(ax = ax1,color = 'teal',label = 'RACMO')
ax11 = ax1.twinx() 
waterfraction_allEles.plot(ax = ax11,marker = 'o',color = 'r',legend = False)
ax11.set_ylim([0, 0.04])
ax1.set_xlabel('DOY')
ax1.set_ylabel('Runoff/Discharge (m3/s)')
ax11.set_ylabel('Water fraction')

ax1.legend(loc='upper left')
ax11.legend(loc='upper right')


#study the correlation between proglacial discharge with SMB runoffs and remotely sensed water fraction
#df_merra_allEles = select_doy_Runoff_from_all(df_runoff,df_merra_allEles)
#df_mar_allEles = select_doy_Runoff_from_all(df_runoff,df_mar_allEles)
#df_racmo_allEles = select_doy_Runoff_from_all(df_runoff,df_racmo_allEles)
#pd_df = select_doy_Runoff_from_all(df_runoff,pd_df)
#ax2.scatter(pd_df,waterfraction_allEles,marker='o',s = 300, c='r')
#ax2.set_ylim([-0.01, 0.04])
#ax21 = ax2.twinx() 
#ax21.scatter(pd_df,df_merra_allEles,marker='o',s = 300, c='b')
#print pd_df['WaterFluxDivers&Temperature(m3/s)'].corr(waterfraction_allEles)


plt.show()


f.savefig(r'E:\proglacial_discharge_2015.tif', format='tif', dpi=450)

#How to plot three y axis?
#f, ax1 = plt.subplots()
#ax3 = ax1.twinx() 
#rspine = ax3.spines['right']
#rspine.set_position(('axes', 1.15))
#ax3.set_frame_on(True)
#ax3.patch.set_visible(False)
#f.subplots_adjust(right=0.7)

#waterfraction_allEles.plot(ax = ax1,marker = 'o',color = 'r')
#pd_df.plot(ax = ax1,color = 'b', legend = False, secondary_y = True)
##df_merra_allEles.plot(ax = ax3,color = 'g')
#df_mar_allEles.plot(ax = ax3,color = 'g')
##df_racmo_allEles.plot(ax = ax3,color = 'g')
